<?php

App::uses('AppBehavior', 'Model/Behavior');

class TemplateBehavior extends AppBehavior
{
    /**
     * Setup this behavior with the specified configuration settings.
     *
     * @param Model $model Model using this behavior
     * @param array $config Configuration settings for $model
     * @return void
     */
    public function setup(Model $Model, $config = array())
    {
        $Model->addBehaviour('EvCustomFields.CustomFieldsData', array('formInject' => true));
    }


    /**
     * get the image / document blocks for the selected template
     * then associate the model wityh the image / docs so we can save
     *
     * @param Model $model Model using this behavior
     * @param array $options Options passed from Model::save().
     * @return mixed False if the operation should abort. Any other result will continue.
     * @see Model::save()
     */
    public function afterValidate(Model $Model)
    {
        $return = parent::afterValidate($Model);

        // when calling from within behavior we need to manually add the Model var in.
        if (isset($Model->data[$Model->alias]['template_id']) && ! empty($Model->data[$Model->alias]['template_id'])) {
            $this->setupAttachmentRelationships($Model, $Model->data[$Model->alias]['template_id']);
        }

        return $return;
    }


    /**
     * given the template id setup the image / docs relationships
     *
     * @param   Model $model Model using this behavior
     * @param   int     templateId
     * @return  void
     */
    public function setupAttachmentRelationships($Model, $templateId)
    {
        $imgBlocks = $this->getAttachmentBlocks('EvTemplates.ImageBlock', $templateId);
        $docBlocks = $this->getAttachmentBlocks('EvTemplates.DocumentBlock', $templateId);

        $this->setupBlock($Model, 'imageSlots', 'ImageBlock', $imgBlocks);
        $this->setupBlock($Model, 'documentSlots', 'DocumentBlock', $docBlocks);

        $Model->initialiseAssociateModels();
    }


    /**
     * setup a single attachment block
     *
     * @param   Model $model Model using this behavior
     * @param   string  Type of block imageSlots|documentSlots
     * @param   string  Model Alias ImageBlock|DocumentBlock
     * @param   array   Array of blocks
     * @return  void
     */
    public function setupBlock($Model, $type, $modelAlias, $blocks)
    {
        if (! empty($blocks)) {
            $slots = array();

            foreach ($blocks as $block) {
                $slots[$block[$modelAlias]['name']] = array(
                    'slots' => $block[$modelAlias]['slots'],
                    'fields' => $block[$modelAlias]['blockFields']
                );
            }
            $Model->$type = $slots;
        }
    }


    /**
     * get image and document blocks
     *
     * @param   string  Attachment type
     * @param   int     templateId
     * @return  array|   array of image blocks / document blocks
     */
    public function getAttachmentBlocks($attachmentType, $templateId)
    {
        $attachmentModel = EvClassRegistry::init($attachmentType);
        $blocks = $attachmentModel->find(
            'all',
            array(
                'conditions' => array(
                    $attachmentModel->alias . '.is_active' => 1,
                    $attachmentModel->alias . '.template_id' => $templateId
                )
            )
        );

        if (! empty($blocks)) {
            return $blocks;
        } else {
            return false;
        }
    }
}
